package JavaProject.Qeditor;
import java.io.*;
import java.util.*;
import java.sql.*;

public class Qloader{
    
    /*******************************************************************************
     * Qloader processes either mySQL or text databases and extracts ArrayLists of *
     * Question objects.  If a mySQL error is detected, the system reverts to the  *
     * default text database, contained withing the JavaProject jar.               *
     * Qloader will never return the same list of Question objects twice.          *
     *******************************************************************************/

    private int charRead;
    private ArrayList tempList;
    private File dataFile;
    private String dbFileName = "JavaProject/Qeditor/javq.db";
    private ResultSet RS;
    private boolean worked = true;
    private int skillLevel;
    
    public Qloader(int skillLevel, String[] data){
        this.skillLevel = skillLevel;
        tempList = new ArrayList();
        dataFile = new File("default");
        String query = "SELECT * FROM qdata WHERE type = " + skillLevel + " or type = " + (skillLevel + 3);
        String login = data[0] + "?user=" + data[1] + "password=" + data[2];
        if(!loadSQLDatabase(query, login)){
            System.out.println("ERROR: mySQL related error detected!");
            System.out.println("ERROR: defaulting to internal database");
            loadTextDatabase();
        } else process(RS);
    }
    
    public Qloader(int skillLevel, File name){
        this.skillLevel = skillLevel;        
        tempList = new ArrayList();
        dataFile = name;
        loadTextDatabase();
    }    
    
    private boolean loadSQLDatabase(String query, String login){
        try{
            Class.forName("org.gjt.mm.mysql.Driver").newInstance();
        } catch (Exception e){
            worked = false;
            System.err.println("Unable to load driver.");
            e.printStackTrace();
        }
        try {
            Connection c = DriverManager.getConnection(login);
            Statement Stmt = c.createStatement();
            RS = Stmt.executeQuery(query);
            Stmt.close();
            c.close();
        } catch (SQLException e){
            worked = false;
            System.out.println("SQLException: " + e.getMessage());
            System.out.println("SQLState:     " + e.getSQLState());
            System.out.println("VendorError:  " + e.getErrorCode());
        } finally {
            return worked;
        }
    }
    
    private void loadTextDatabase(){
        try {
            BufferedReader bufRead;
            String bulkText = "";
            if (dataFile.getName() != "default"){
                bufRead = new BufferedReader(new FileReader(dataFile));
            } else {
                bufRead = new BufferedReader(new InputStreamReader(ClassLoader.getSystemResourceAsStream(dbFileName)));
            }
            String currentLine = bufRead.readLine();
            while (currentLine != null){
                bulkText += currentLine + '\r';
                currentLine = bufRead.readLine();
            }
            process(bulkText);
            bufRead.close();
        } catch (IOException e){
            System.out.println("ERROR: File related error detected!");
            e.printStackTrace();
        };
    }    
    
    private void process(ResultSet rs){
        try{
            while(rs.next()){
                int type = rs.getInt("type");
                String questionText = rs.getString("qText");
                Answer[] tempAz = new Answer[4];
                tempAz[0] = new Answer(rs.getString("ans1"), rs.getString("chk1"));
                tempAz[1] = new Answer(rs.getString("ans2"), rs.getString("chk2"));
                tempAz[2] = new Answer(rs.getString("ans3"), rs.getString("chk3"));
                tempAz[3] = new Answer(rs.getString("ans4"), rs.getString("chk4"));
                tempList.add(new Question(type, questionText, tempAz));
            }
            rs.close();
        } catch (SQLException e){
            System.out.println("SQLException: " + e.getMessage());
            System.out.println("SQLState:     " + e.getSQLState());
            System.out.println("VendorError:  " + e.getErrorCode());
        }
    }
    
    private void process(String temp){
        ArrayList[] qList = {new ArrayList(), new ArrayList(), new ArrayList()};
        charRead = 0;
        for (int i = charRead; i < temp.length(); i++){
            boolean diff = true;
            int difficultyLevel = 0;
            while (diff) {
                char j = temp.charAt(charRead++);
                int k = ((int)j - 48);
                if ((k<6)&&(k>-1)){
                    difficultyLevel = k;
                    diff = false;}
            }
            charRead++;
            String questionText = getNext(temp);
            Answer[] tempAz = new Answer[4];
            for (int j = 0; j < 4; j++){
                String answerText = getNext(temp);
                String answerBool = getNext(temp);
                tempAz[j] = new Answer(answerText, answerBool);
            }
            int select = difficultyLevel;
            if (select > 2){select-=3;}
            ArrayList currentList = qList[select];
            currentList.add(new Question(difficultyLevel, questionText, tempAz));
            i = charRead;
        }
        tempList = qList[skillLevel];
    } 

    private String getNext(String temp){
        //Helper method for process(String) method
        String tempS = "";
        while(temp.charAt(charRead) != '#'){
                tempS += temp.charAt(charRead++);
            }
        charRead++;
        return tempS;
    }
    
    public ArrayList getQuestions(int quantity){
        // returns a random selection of Questions
        ArrayList newList = new ArrayList();
        for(int i = 0; i < quantity; i++){
            int j = (int)(Math.random() * tempList.size());
            Question temp = (Question)tempList.get(j);
            newList.add(temp);
            tempList.remove(j);
        }
        return newList;
    }
    
    public boolean enoughQz(int i){
        // used to check if there are enough Questions to run a test with
        if (i > tempList.size()){return false;}
        return true;
    }
}