package JavaProject.Test;

import JavaProject.Qeditor.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;
import java.text.NumberFormat;
import JavaProject.Gui.*;

public class Asker implements Runnable, ActionListener {
    
    /****************************************************************************
     * This class is a Thread that controls the handling of the question asking *
     * process.                                                                 *
     ****************************************************************************/
    
    private User user;
    private ArrayList questions;
    private Question currentQuestion;
    private int Qasked;
    private JLabel qLabel;
    private NumberFormat nf;
    private ADisplay aDisplay;
    private QDisplay qDisplay;
    private String[] act = {"0", "1", "2", "3", "0"};
    private String currentMode;
    private JButton[] buttons;
    
    public Asker(User user, Qloader loader){
//        System.out.println("in Asker module");
        this.user = user;
        nf = NumberFormat.getInstance();
        nf.setMinimumIntegerDigits(2);
        nf.setMaximumFractionDigits(0);        
        aDisplay = user.getGUI().getADisplay();
        qDisplay = user.getGUI().getQDisplay();
        questions = loader.getQuestions(user.getNumOfQuestions());
        Qasked = 0;
    }
    
    public void run(){
        setupButtons();
        nextQ();
    }
    
    public void setupButtons(){
        buttons = new JButton[5];
        System.arraycopy(user.getGUI().getControlDisplay().getButtons(),0, buttons, 0, 3);
        System.arraycopy(user.getGUI().getLogDisplay().getButtons(), 0, buttons, 3, 2);
        for (int i = 0; i < buttons.length; i++){
            buttons[i].setActionCommand(act[i]); 
            buttons[i].addActionListener(this);
        }
    }
    
    public void nextQ(){
        // Fetches the next Question provided the test isn't over yet
        if(!user.getEnd()){
            currentQuestion = (Question)questions.get(Qasked++);
            qDisplay.getQLabel().setText(nf.format(Qasked) + " of " + user.getNumOfQuestions());
            ask(currentQuestion);
        } else theEnd();
    }
    
    public void ask(Question q){
        // Displays the appropriate answer interface
        int i = q.getDifficulty();
        if (i<3){currentMode = "multi";}
        else if (i>2){currentMode="input";}
        aDisplay.setMode(currentMode);
        qDisplay.setQText(q.getQuestion());
        if (currentMode=="multi"){
            aDisplay.setMultAnswers(q.getAnswers());
        }    
    }

    public boolean checkAnswer(Question q){
        boolean temp = true;
        if (currentMode == "multi"){
            MultAnswerChecker check = new MultAnswerChecker(
                q.getValidity(), aDisplay.getMultAnswer());
            temp = check.result();
        } else if (currentMode == "input"){
            InpAnswerChecker check = new InpAnswerChecker(
                q.getAnswers(), aDisplay.getInpAnswer());
            temp = check.result();
        } return temp;
    }
    
    public void theEnd(){
        user.setEnd();
        user.getGUI().hideTab(1, false, 2);
        user.getGUI().getLogDisplay().setResults(user.getResults());
     }
    
    public void cleanup(){
        for (int i = 0; i < buttons.length; i++){
            buttons[i].removeActionListener(this);}
        user.getGUI().hideTab(2, false, 0);
        user.reset(); user = null;
    }
    
    public void actionPerformed(ActionEvent e){
        switch(Integer.parseInt(e.getActionCommand())){
            case 0:
                //quit
                System.exit(0);
                break;
            case 1:
                //bail
                user.bailOut();
                theEnd();
                break;
            case 2:
                //next
                user.updateLog(currentQuestion, checkAnswer(currentQuestion));
                if(currentQuestion.getDifficulty()>2){aDisplay.clearInpAnswer();}
                if(user.getNumOfQuestions() == user.getNumAnswered()){
                    theEnd();
                } else nextQ();
                break;
            case 3:
                //new test
                cleanup();
        }
    }
}